"""
The Exceptions test module.

This module contains tests for Exceptions.
"""
import pytest

from himon.exceptions import AuthenticationError, ServiceError
from himon.league_of_comic_geeks import LeagueofComicGeeks


def test_unauthorized():
    """Test generating an AuthenticationError."""
    session = LeagueofComicGeeks(api_key="Invalid", client_id="Invalid", cache=None)
    with pytest.raises(AuthenticationError):
        session.comic(comic_id=1)


def test_not_found(session: LeagueofComicGeeks):
    """Test a 404 Not Found raises a ServiceError."""
    with pytest.raises(ServiceError):
        session._get_request(endpoint="/invalid")


def test_timeout(league_of_comic_geeks_api_key: str, league_of_comic_geeks_client_id: str):
    """Test a TimeoutError for slow responses."""
    session = LeagueofComicGeeks(
        api_key=league_of_comic_geeks_api_key,
        client_id=league_of_comic_geeks_client_id,
        timeout=0.1,
        cache=None,
    )
    with pytest.raises(ServiceError):
        session.comic(comic_id=1)



"""
The conftest module.

This module contains pytest fixtures.
"""
import os

import pytest

from himon.league_of_comic_geeks import LeagueofComicGeeks
from himon.sqlite_cache import SQLiteCache


@pytest.fixture(scope="session")
def league_of_comic_geeks_api_key():
    """Set the League of Comic Geeks API key fixture."""
    return os.getenv("LEAGUE_OF_COMIC_GEEKS__API_KEY", default="INVALID")


@pytest.fixture(scope="session")
def league_of_comic_geeks_client_id():
    """Set the League of Comic Geeks Client Id fixture."""
    return os.getenv("LEAGUE_OF_COMIC_GEEKS__CLIENT_ID", default="INVALID")


@pytest.fixture(scope="session")
def session(
    league_of_comic_geeks_api_key: str, league_of_comic_geeks_client_id: str
) -> LeagueofComicGeeks:
    """Set the Himon session fixture."""
    return LeagueofComicGeeks(
        api_key=league_of_comic_geeks_api_key,
        client_id=league_of_comic_geeks_client_id,
        cache=SQLiteCache("tests/cache.sqlite", expiry=None),
    )



"""
The init test module.

This module contains tests for himon.schemas.__init__ functions.
"""
import pytest

from himon.schemas import to_bool, to_optional_float, to_optional_int, to_optional_str


def test_optional_int():
    """Test converting an Int to an Optional[Int]."""
    output = to_optional_int(12)
    assert output == 12


def test_optional_int_str():
    """Test converting a Str to an Optional[Int]."""
    output = to_optional_int("12")
    assert output == "12"


def test_optional_int_invalid():
    """Test converting an invalid value to an Optional[Int]."""
    with pytest.raises(ValueError):
        to_optional_int("Not Integer")


def test_optional_int_zero():
    """Test converting a 0 to an Optional[Int]."""
    output = to_optional_int("0")
    assert output is None


def test_optional_int_none():
    """Test converting a None to an Optional[Int]."""
    output = to_optional_int(None)
    assert output is None


def test_optional_float():
    """Test converting a Float to an Optional[Float]."""
    output = to_optional_float(1.23)
    assert output == "1.23"


def test_optional_float_str():
    """Test converting a Str to an Optional[Float]."""
    output = to_optional_float("1.23")
    assert output == "1.23"


def test_optional_float_double_decimal():
    """Test converting a Str with a double decimal to an Optional[Float]."""
    output = to_optional_float("1..23")
    assert output == "1.23"


def test_optional_float_invalid():
    """Test converting an invalid value to an Optional[Float]."""
    with pytest.raises(ValueError):
        to_optional_float("Not Float")


def test_optional_float_zero():
    """Test converting a 0 to an Optional[Float]."""
    output = to_optional_float("0")
    assert output is None


def test_optional_float_none():
    """Test converting a None to an Optional[Float]."""
    output = to_optional_float(None)
    assert output is None


def test_optional_str():
    """Test converting a Str to an Optional[Str]."""
    output = to_optional_str("String value")
    assert output == "String value"


def test_optional_str_none():
    """Test converting a None to an Optional[Str]."""
    output = to_optional_str(None)
    assert output is None


def test_bool_true():
    """Test converting a Str to a Bool."""
    output = to_bool("1")
    assert output is True


def test_bool_false():
    """Test converting a Str to a Bool."""
    output = to_bool("0")
    assert output is False


def test_bool_invalid():
    """Test converting an invalid value to a Bool."""
    with pytest.raises(ValueError):
        to_bool("2")



"""himon package entry file."""
__version__ = "0.2.1"
__all__ = ["__version__", "get_cache_root"]

from pathlib import Path


def get_cache_root() -> Path:
    """
    Create and return the path to the cache for Himon.

    Returns:
        The path to the Himon cache
    """
    folder = Path.home() / ".cache" / "himon"
    folder.mkdir(parents=True, exist_ok=True)
    return folder



"""
The SQLiteCache module.

This module provides the following classes:

- SQLiteCache
"""
import json
import sqlite3
from datetime import date, timedelta
from typing import Any, Dict, Optional

from himon import get_cache_root


class SQLiteCache:
    """
    The SQLiteCache object to cache search results from League of Comic Geeks.

    Args:
        path: Path to database.
        expiry: How long to keep cache results, in days.

    Attributes:
        expiry (Optional[timedelta]): How long to keep cache results, in days.
        con (sqlite3.Connection): Database connection
        cur (sqlite3.Cursor): Database cursor
    """

    def __init__(
        self,
        path: str = get_cache_root() / "cache.sqlite",
        expiry: Optional[int] = 14,
    ):
        self.expiry = timedelta(days=expiry) if expiry else None
        self.con = sqlite3.connect(path)
        self.cur = self.con.cursor()
        self.cur.execute("CREATE TABLE IF NOT EXISTS queries (query, response, date_added);")
        self.delete()

    def select(self, query: str) -> Dict[str, Any]:
        """
        Retrieve data from the cache database.

        Args:
            query: Search string
        Returns:
            Empty dict or select results.
        """
        if self.expiry:
            self.cur.execute(
                "SELECT response FROM queries WHERE query = ? and date_added > ?;",
                (query, (date.today() - self.expiry).isoformat()),
            )
        else:
            self.cur.execute("SELECT response FROM queries WHERE query = ?;", (query,))
        results = self.cur.fetchone()
        if results:
            return json.loads(results[0])
        return {}

    def insert(self, query: str, response: str):
        """
        Insert data into the cache database.

        Args:
            query: Search string
            response: Data to save
        """
        self.cur.execute(
            "INSERT INTO queries (query, response, date_added) VALUES (?, ?, ?);",
            (query, json.dumps(response), date.today().isoformat()),
        )
        self.con.commit()

    def delete(self):
        """Remove all expired data from the cache database."""
        if not self.expiry:
            return
        self.cur.execute(
            "DELETE FROM queries WHERE date_added <= ?;",
            ((date.today() - self.expiry).isoformat(),),
        )
        self.con.commit()
